@@ -20,18 +20,14 @@
 #ifndef ENTITY_H
 #define ENTITY_H
 
-#include "countquerybuilder.h"
-#include "../akonadiprivate_export.h"
-
 #include <QtCore/Qt>
 #include <QtCore/QDateTime>
 #include <QtCore/QDebug>
 #include <QtCore/QString>
 #include <QtCore/QStringList>
-#include <QtCore/QVariant>
-#include <QtSql/QSqlDatabase>
-#include <QtSql/QSqlQuery>
-#include <QtSql/QSqlError>
+
+class QVariant;
+class QSqlDatabase;
 
 namespace Akonadi {
 
@@ -39,19 +35,22 @@ namespace Akonadi {
   Base class for classes representing database records. It also contains
   low-level data access and manipulation template methods.
 */
-class AKONADIPRIVATE_EXPORT Entity
+class Entity
 {
   public:
     typedef qint64 Id;
+
+  protected:
     qint64 id() const;
     void setId( qint64 id );
 
     bool isValid() const;
 
-    template <typename T> static QString joinByName( const QList<T> &list, const QString &sep )
+  public:
+    template <typename T> static QString joinByName( const QVector<T> &list, const QString &sep )
     {
       QStringList tmp;
-      foreach ( T t, list )
+      Q_FOREACH( const T & t, list )
         tmp << t.name();
       return tmp.join( sep );
     }
@@ -63,20 +62,7 @@ class AKONADIPRIVATE_EXPORT Entity
     */
     template <typename T> inline static int count( const QString &column, const QVariant &value )
     {
-      QSqlDatabase db = database();
-      if ( !db.isOpen() )
-        return -1;
-
-      CountQueryBuilder builder( T::tableName() );
-      builder.addValueCondition( column, Query::Equals, value );
-
-      if ( !builder.exec() ) {
-        qDebug() << "Error during counting records in table" << T::tableName()
-            << builder.query().lastError().text();
-        return -1;
-      }
-
-      return builder.result();
+      return Entity::countImpl( T::tableName(), column, value );
     }
 
     /**
@@ -84,19 +70,7 @@ class AKONADIPRIVATE_EXPORT Entity
     */
     template <typename T> inline static bool remove( const QString &column, const QVariant &value )
     {
-      QSqlDatabase db = database();
-      if ( !db.isOpen() )
-        return false;
-
-      QueryBuilder builder( T::tableName(), QueryBuilder::Delete );
-      builder.addValueCondition( column, Query::Equals, value );
-
-      if ( !builder.exec() ) {
-        qDebug() << "Error during deleting records from table"
-            << T::tableName() << builder.query().lastError().text();
-        return false;
-      }
-      return true;
+      return Entity::removeImpl( T::tableName(), column, value );
     }
 
     /**
@@ -106,23 +80,7 @@ class AKONADIPRIVATE_EXPORT Entity
      */
     template <typename T> inline static bool relatesTo( qint64 leftId, qint64 rightId )
     {
-      QSqlDatabase db = database();
-      if ( !db.isOpen() )
-        return false;
-
-      CountQueryBuilder builder( T::tableName() );
-      builder.addValueCondition( T::leftColumn(), Query::Equals, leftId );
-      builder.addValueCondition( T::rightColumn(), Query::Equals, rightId );
-
-      if ( !builder.exec() ) {
-        qDebug() << "Error during counting records in table" << T::tableName()
-            << builder.query().lastError().text();
-        return false;
-      }
-
-      if ( builder.result() > 0 )
-        return true;
-      return false;
+      return Entity::relatesToImpl( T::tableName(), T::leftColumn(), T::rightColumn(), leftId, rightId );
     }
 
     /**
@@ -132,30 +90,7 @@ class AKONADIPRIVATE_EXPORT Entity
     */
     template <typename T> inline static bool addToRelation( qint64 leftId, qint64 rightId )
     {
-      QSqlDatabase db = database();
-      if ( !db.isOpen() )
-        return false;
-
-      QString statement = QLatin1String("INSERT INTO ");
-      statement.append( T::tableName() );
-      statement.append( QLatin1String(" ( ") );
-      statement.append( T::leftColumn() );
-      statement.append( QLatin1String(" , ") );
-      statement.append( T::rightColumn() );
-      statement.append( QLatin1String(" ) VALUES ( :left, :right )") );
-
-      QSqlQuery query( db );
-      query.prepare( statement );
-      query.bindValue( QLatin1String(":left"), leftId );
-      query.bindValue( QLatin1String(":right"), rightId );
-
-      if ( !query.exec() ) {
-        qDebug() << "Error during adding a record to table" << T::tableName()
-          << query.lastError().text();
-        return false;
-      }
-
-      return true;
+      return Entity::addToRelationImpl( T::tableName(), T::leftColumn(), T::rightColumn(), leftId, rightId );
     }
 
     /**
@@ -165,21 +100,7 @@ class AKONADIPRIVATE_EXPORT Entity
     */
     template <typename T> inline static bool removeFromRelation( qint64 leftId, qint64 rightId )
     {
-      QSqlDatabase db = database();
-      if ( !db.isOpen() )
-        return false;
-
-      QueryBuilder builder( T::tableName(), QueryBuilder::Delete );
-      builder.addValueCondition( T::leftColumn(), Query::Equals, leftId );
-      builder.addValueCondition( T::rightColumn(), Query::Equals, rightId );
-
-      if ( !builder.exec() ) {
-        qDebug() << "Error during removing a record from relation table" << T::tableName()
-          << builder.query().lastError().text();
-        return false;
-      }
-
-      return true;
+      return Entity::removeFromRelationImpl( T::tableName(), T::leftColumn(), T::rightColumn(), leftId, rightId );
     }
 
     enum RelationSide {
@@ -194,40 +115,67 @@ class AKONADIPRIVATE_EXPORT Entity
     */
     template <typename T> inline static bool clearRelation( qint64 id, RelationSide side = Left )
     {
-      QSqlDatabase db = database();
-      if ( !db.isOpen() )
-        return false;
-
-      QueryBuilder builder( T::tableName(), QueryBuilder::Delete );
-      switch ( side ) {
-        case Left:
-          builder.addValueCondition( T::leftColumn(), Query::Equals, id );
-          break;
-        case Right:
-          builder.addValueCondition( T::rightColumn(), Query::Equals, id );
-          break;
-        default:
-          qFatal("Invalid enum value");
-      }
-      if ( !builder.exec() ) {
-        qDebug() << "Error during clearing relation table" << T::tableName()
-            << "for id" << id << builder.query().lastError().text();
-        return false;
-      }
-
-      return true;
+      return Entity::clearRelationImpl( T::tableName(), T::leftColumn(), T::rightColumn(), id, side );
     }
 
   protected:
     Entity();
-    Entity( qint64 id );
+    explicit Entity( qint64 id );
+    ~Entity();
 
+  private:
+    static int countImpl( const QString & tableName, const QString & column, const QVariant & value );
+    static bool removeImpl( const QString & tableName, const QString & column, const QVariant & value );
+    static bool relatesToImpl( const QString & tableName, const QString & leftColumn, const QString & rightColumn, qint64 leftId, qint64 rightId );
+    static bool addToRelationImpl( const QString & tableName, const QString & leftColumn, const QString & rightColumn, qint64 leftId, qint64 rightId );
+    static bool removeFromRelationImpl( const QString & tableName, const QString & leftColumn, const QString & rightColumn, qint64 leftId, qint64 rightId );
+    static bool clearRelationImpl( const QString & tableName, const QString & leftColumn, const QString & rightColumn, qint64 id, RelationSide side );
 
   private:
     static QSqlDatabase database();
     qint64 m_id;
 };
 
+namespace _detail {
+
+    /*!
+      Binary predicate to sort collections of Entity subclasses by
+      their id.
+
+      Example for sorting:
+      \code
+      std::sort( coll.begin(), coll.end(), _detail::ById<std::less>() );
+      \endcode
+
+      Example for finding by id:
+      \code
+      // linear:
+      std::find_if( coll.begin(), coll.end(), bind( _detail::ById<std::equal_to>(), _1, myId ) );
+      // binary:
+      std::lower_bound( coll.begin(), coll.end(), myId, _detail::ById<std::less>() );
+      \end
+    */
+    template <template <typename U> class Op>
+    struct ById {
+        typedef bool result_type;
+        bool operator()( Entity::Id lhs, Entity::Id rhs ) const {
+            return Op<Entity::Id>()( lhs, rhs );
+        }
+        template <typename E>
+        bool operator()( const E & lhs, const E & rhs ) const {
+            return this->operator()( lhs.id() , rhs.id() );
+        }
+        template <typename E>
+        bool operator()( const E & lhs, Entity::Id rhs ) const {
+            return this->operator()( lhs.id(), rhs );
+        }
+        template <typename E>
+        bool operator()( Entity::Id lhs, const E & rhs ) const {
+            return this->operator()( lhs, rhs.id() );
+        }
+    };
+}
+
 }
 
 
