@@ -78,7 +78,7 @@ class Tracer : public QObject, public TracerInterface
      *                   is retrieved.
      * @param msg A message specific string.
      */
-    virtual void connectionInput( const QString &identifier, const QString &msg );
+    virtual void connectionInput( const QString &identifier, const QByteArray &msg );
 
     /**
      * This method is called whenever the akonadi server sends some data out to a client.
@@ -87,7 +87,7 @@ class Tracer : public QObject, public TracerInterface
      *                   data is send.
      * @param msg A message specific string.
      */
-    virtual void connectionOutput( const QString &identifier, const QString &msg );
+    virtual void connectionOutput( const QString &identifier, const QByteArray &msg );
 
     /**
      * This method is called whenever a dbus signal is emitted on the bus.
